require("lib.class")
lume = require("lib.lume")
require("util")

Entity = class()

function Entity:_init(x, y, sizeX, sizeY, offX, offY)
  self.x = x or 0
  self.vx = 0
  self.y = y or 0
  self.vy = 0
  self.scaleX = 1
  self.scaleY = 1
  self.friction = 0.9
  self.terminalX = 450
  self.terminalY = 450
  self.rotation = 0
  self.gravity = 1600
  self.sprite = nil
  self.offX = offX or 0
  self.offY = offY or 0
  self.sizeX = sizeX or 12
  self.sizeY = sizeY or 20
  self.health = 0
  self.id = "Entity"

  self.tint = {}
  self.tintFrames = 0
  self.hitby = 0
  self.dead = false
  self.isenemy = false
end

function Entity:Update(dt)
  self.poop = false

  self.x = self.x + self.vx * dt
  self.y = self.y + self.vy * dt

  self.x = math.floor(self.x + 0.5)
  self.y = math.floor(self.y + 0.5)

  if self.shape ~= nil then
    self.shape:moveTo(self.x + self.offX, self.y + self.offY)

    local coll = false
    for shape, delta in pairs(World:collisions(self.shape)) do
      coll = true
      if shape.me ~= nil and self.OnCollision then
        if self:OnCollision(shape.me) then
          self.x = self.x + delta.x
          self.y = self.y + delta.y

          if delta.y < 0 then self.onGround = true self.vy = 0
          elseif delta.y > 0 then self.onGround = false self.vy = 0 end

          if delta.x ~= 0 then self.vx = 0 end
        end
      elseif shape.solid == nil or shape.solid == true then
        self.x = self.x + delta.x
        self.y = self.y + delta.y

        if delta.y < 0 then self.onGround = true self.vy = 0
        elseif delta.y > 0 then self.onGround = false self.vy = 0 end

        if delta.x ~= 0 then self.vx = 0 end
      end
    end
  end
  self.x = math.floor(self.x + 0.5)
  self.y = math.floor(self.y + 0.5)

  self.vx = self.vx * (self.friction - 0.07)
  self.vy = self.vy * (self.friction - 0.07)

  if math.abs(self.vx) > self.terminalX then
    self.vx = lume.sign(self.vx) * self.terminalX
  end

  if math.abs(self.vy) > self.terminalY then
    self.vy = lume.sign(self.vy) * self.terminalY
  end

  if self.sprite ~= nil then
    self.sprite:Update(dt)
  end
end

function Entity:Register()
  self.shape = World:rectangle((self.x - self.sizeX / 2) + self.offX, (self.y - self.sizeY / 2) + self.offY, self.sizeX, self.sizeY)
  self.shape.me = self
  self.shape.solid = false
end

function Entity:MoveTo(x, y)
  self.x = x
  self.y = y
  self.x = math.floor(self.x + 0.5)
  self.y = math.floor(self.y + 0.5)

  self.shape:moveTo(self.x + self.offX, self.y + self.offY)

  for shape, delta in pairs(World:collisions(self.shape)) do
    if shape.me ~= nil and self.OnCollision then
      if self:OnCollision(shape.me) then
        self.x = self.x + delta.x
        self.y = self.y + delta.y

        if delta.y < 0 then self.onGround = true self.vy = 0
        elseif delta.y > 0 then self.onGround = false self.vy = 0 end

        if delta.x ~= 0 then self.vx = 0 end
      end
    elseif shape.solid == nil or shape.solid == false then
      self.x = self.x + delta.x
      self.y = self.y + delta.y

      if delta.y < 0 then self.onGround = true self.vy = 0
      elseif delta.y > 0 then self.onGround = false self.vy = 0 end

      if delta.x ~= 0 then self.vx = 0 end
    end
  end
end

function Entity:Draw()
  if self.sprite ~= nil then
  --  self.sprite:Draw(self.x + self.offX, self.y + self.offY, self.rotation, self.scaleX, self.scaleY)
  end
  if self.shape ~= nil and debug.colliders == true then
    self.shape:draw('line')
  end
end

function Entity:Remove()
  if self.shape then
    World:remove(self.shape)
  end
end
