local lume = require("lib.lume")
local HC = require("lib.hardoncollider")

require("sprite")    require("entity")  require("tilemap")
require("collision") require("camera")  require("map")
require("gfx")

function Setup()
  ABILITIES = {
    [CIRCLE] = function(self)
      self.atimer = self.atimer - 1

      if love.mouse.isDown("r") and self.atimer <= 0 then
        local dx, dy = love.mouse.getPosition()
        dx = (dx - 1024 / 2) / 2  - self.x
        dy = (dy - 768 / 2) / 2 - self.y
        local dir = math.atan2(dy, dx)
        local dirX = math.cos(dir)
        local dirY = math.sin(dir)
        self.vx = dirX * 1400
        self.vy = dirY * 1400
        self.atimer = 45
      end

      if self.atimer > 35 then
        TRAIL[#TRAIL + 1] = {self.x, self.y, 5}
        self.candie = false
      else
        self.candie = true
      end
    end,
    [SQUARE] = function(self)
      self.atimer = self.atimer - 1

      if love.mouse.isDown("r") and self.atimer <= 0 then
        local dx, dy = love.mouse.getPosition()
        dx = (dx - 1024 / 2) / 2  - self.x
        dy = (dy - 768 / 2) / 2 - self.y
        local dir = math.atan2(dy, dx)
        local dist = 40
        for i = -5, 5 do
          local dirX = math.cos(dir + i / 5)
          local dirY = math.sin(dir + i / 5)
          local dist = 40 - math.abs(i) * 2
          WALL[#WALL + 1] = {
            self.x + dirX * dist,
            self.y + dirY * dist,
            20
          }
        end

        self.atimer = 90
      end
    end,
    [TRIANGLE] = function(self)
      self.atimer = self.atimer - 1

      if love.mouse.isDown("r") and self.atimer <= 0 then
        for i = 0, 5 do
          SHOOT[TRIANGLE](self, 0.3 * ((math.random() - 0.5) * 2))
        end
        self.atimer = 120
      end
    end
  }

  SHOTS = {}

  SHOOT = {
    [CIRCLE] = function(self)
      self.shottimer = self.shottimer - 1
      if love.mouse.isDown('l') and self.shottimer <= 0 then
        local dx, dy = love.mouse.getPosition()
        dx = (dx - 1024 / 2) / 2  - self.x
        dy = (dy - 768 / 2) / 2 - self.y
        local dir = math.atan2(dy, dx)
        local dist = 12

        gameCamera:Shake(10)

        for i = 0, 15 do
          local rot = dir + 0.3 * ((math.random() - 0.5) * 2)
          SHOTS[#SHOTS + 1] = {
            damage = 2,
            speed = 300 + 700 * math.random(),
            rot = rot,
            x = self.x + dist * math.cos(rot),
            y = self.y + dist * math.sin(rot),
            sprite = Copy(SPRITES.CircleShot),
            size = 12,
            alive = true,
            Update = function(self, dt)
              self.speed = self.speed * 0.85
              self.x = self.x + math.cos(self.rot) * self.speed * dt
              self.y = self.y + math.sin(self.rot) * self.speed * dt
              if self.speed <= 10 then
                self.alive = false
              end
            end,
            Draw = function(self)
              if self.speed <= 100 then
                love.graphics.setColor(255, 255, 255, 255 * self.speed / 100)
              end
              self.sprite:Draw(lume.round(self.x), lume.round(self.y), self.rot)
              love.graphics.setColor(255, 255, 255, 255)
            end
          }
        end
        love.audio.play(SOUNDS[CIRCLE])

        self.shottimer = 45
      end
    end,
    [SQUARE] = function(self)
      self.shottimer = self.shottimer - 1
      if love.mouse.isDown('l') and self.shottimer <= 0 then
        local dx, dy = love.mouse.getPosition()
        dx = (dx - 1024 / 2) / 2  - self.x
        dy = (dy - 768 / 2) / 2 - self.y
        local dir = math.atan2(dy, dx)
        local dist = 12

        gameCamera:Shake(2)

        local rot = dir + 0.05 * ((math.random() - 0.5) * 2)
        SHOTS[#SHOTS + 1] = {
          damage = 3,
          speed = 1000,
          rot = rot,
          dist = 0,
          size = 8,
          x = self.x + dist * math.cos(rot),
          y = self.y + dist * math.sin(rot),
          sprite = Copy(SPRITES.SquareShot),
          alive = true,
          Update = function(self, dt)
            self.dist = self.dist + math.abs(math.cos(self.rot) * self.speed * dt) + math.abs(math.sin(self.rot) * self.speed * dt)
            self.x = self.x + math.cos(self.rot) * self.speed * dt
            self.y = self.y + math.sin(self.rot) * self.speed * dt
            if self.dist >= 300 then
              self.alive = false
            end
          end,
          Draw = function(self)
            if self.dist >= 250 then
              love.graphics.setColor(255, 255, 255, 255 * (self.dist - 250) / 50)
            end
            self.sprite:Draw(lume.round(self.x), lume.round(self.y), self.rot)
            love.graphics.setColor(255, 255, 255, 255)
          end
        }
        love.audio.play(SOUNDS[SQUARE])

        self.shottimer = 7
      end
    end,
    [TRIANGLE] = function(self, offset)
      if not offset then self.shottimer = self.shottimer - 1 end
      if (love.mouse.isDown('l') and self.shottimer <= 0) or offset then
        local dx, dy = love.mouse.getPosition()
        dx = (dx - 1024 / 2) / 2  - self.x
        dy = (dy - 768 / 2) / 2 - self.y
        local dir = math.atan2(dy, dx)
        local dist = 350

        if not offset then gameCamera:Shake(15) else gameCamera:Shake(3) end

        local rot = dir + (offset or 0)

        local ignore = {}

        for i = 0, 90 do
          for it, v in ipairs(FOES) do
            if Hit(self.x + (i * 8) * math.cos(rot), self.y + (i * 8) * math.sin(rot), 8, v.x, v.y, v.size) and not Contains(ignore, v) then
              v.health = v.health - (offset and 5 or 10)
              Emitt(v.x, v.y, 1 + math.ceil(math.random() * 3))
              ignore[#ignore + 1] = v
              love.audio.play(SOUNDS.Hit)
            end
          end
        end
        love.audio.play(SOUNDS[TRIANGLE])
        SHOTS[#SHOTS + 1] = {
          damage = 10,
          rot = rot,
          time = 0,
          x = self.x + dist * math.cos(rot),
          y = self.y + dist * math.sin(rot),
          sprite = Copy(SPRITES.TriangleShot),
          alive = true,
          Update = function(self, dt)
            self.time = self.time + 1
            if self.time >= 25 then
              self.alive = false
            end
          end,
          Draw = function(self)
            if self.time >= 10 then
              love.graphics.setColor(255, 255, 255, 255 * (1 - (self.time - 10) / 15))
            end
            self.sprite:Draw(lume.round(self.x), lume.round(self.y), self.rot)
            love.graphics.setColor(255, 255, 255, 255)
          end
        }

        self.shottimer = 45
      end
    end
  }

  ESHOTS = {}

  ENEMIES = {
    -- Basic circle
    {
      shottimer = 0,
      x = 0,
      y = 0,
      vx = 0,
      vy = 0,
      size = 16,
      health = 10,
      sprite = SPRITES.Enemy1,
      Update = function(self, dt)
        self.shottimer = self.shottimer - 1

        self.vx = self.vx * 0.9
        self.vy = self.vy * 0.9

        self.x = self.x + self.vx * dt
        self.y = self.y + self.vy * dt

        if lume.distance(self.x, self.y, player.x, player.y) > 100 or not OnScreen(self.x, self.y) then
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)
          self.vx = self.vx + math.cos(dir) * 400 * dt
          self.vy = self.vy + math.sin(dir) * 400 * dt
        elseif lume.distance(self.x, self.y, player.x, player.y) < 25 then
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)
          self.vx = self.vx + math.cos(dir) * 400 * dt
          self.vy = self.vy + math.sin(dir) * 400 * dt
        else
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)
          self.vx = math.cos(dir + (math.pi / 2)) * 100
          self.vy = math.sin(dir + (math.pi / 2)) * 100

          if self.shottimer <= 0 then
            ESHOTS[#ESHOTS + 1] = {
              sprite = Copy(SPRITES.Enemy1Shot),
              x = self.x,
              y = self.y,
              alive = true,
              size = 8,
              vx = math.cos(dir + 0.2 * (math.random() - 0.5) * 2) * 200,
              vy = math.sin(dir + 0.2 * (math.random() - 0.5) * 2) * 200,
              shape = CIRCLE,
              Update = function(self, dt)
                self.x = self.x + self.vx * dt
                self.y = self.y + self.vy * dt
                if not OnScreen(self.x, self.y) then self.alive = false end
              end,
              Draw = function(self)
                self.sprite:Draw(math.floor(self.x), math.floor(self.y))
              end
            }
            self.shottimer = 20
          end
        end
      end,
      Draw = function(self)
        self.sprite:Draw(math.floor(self.x), math.floor(self.y))
      end
    },
    -- Square
    {
      shottimer = 0,
      x = 0,
      y = 0,
      vx = 0,
      vy = 0,
      size = 16,
      health = 10,
      sprite = SPRITES.Enemy2,
      Shoot = function(self, dir)
        ESHOTS[#ESHOTS + 1] = {
          sprite = Copy(SPRITES.Enemy2Shot),
          alive = true,
          x = self.x,
          y = self.y,
          size = 8,
          vx = math.cos(dir + 0.1 * (math.random() - 0.5) * 2) * 200,
          vy = math.sin(dir + 0.1 * (math.random() - 0.5) * 2) * 200,
          shape = SQUARE,
          Update = function(self, dt)
            self.x = self.x + self.vx * dt
            self.y = self.y + self.vy * dt
            if not OnScreen(self.x, self.y) then self.alive = false end
          end,
          Draw = function(self)
            self.sprite:Draw(math.floor(self.x), math.floor(self.y))
          end
        }
      end,
      Update = function(self, dt)
        self.shottimer = self.shottimer - 1

        self.vx = self.vx * 0.9
        self.vy = self.vy * 0.9

        self.x = self.x + self.vx * dt
        self.y = self.y + self.vy * dt

        if lume.distance(self.x, self.y, player.x, player.y) > 150 or not OnScreen(self.x, self.y) then
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)
          self.vx = self.vx + math.cos(dir) * 400 * dt
          self.vy = self.vy + math.sin(dir) * 400 * dt
        elseif lume.distance(self.x, self.y, player.x, player.y) < 40 then
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)
          self.vx = self.vx + math.cos(dir) * 400 * dt
          self.vy = self.vy + math.sin(dir) * 400 * dt
        else
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)

          if self.shottimer <= 0 then
            CB.delay(function() self:Shoot(dir-0.2) end, 0)
              :after(function() self:Shoot(dir-0.1) end, 0.1)
              :after(function() self:Shoot(dir) end, 0.1)
              :after(function() self:Shoot(dir+0.1) end, 0.1)
              :after(function() self:Shoot(dir+0.2) end, 0.1)

            self.shottimer = 120
          end
        end
      end,
      Draw = function(self)
        self.sprite:Draw(math.floor(self.x), math.floor(self.y))
      end
    }, {
      shottimer = 0,
      x = 0,
      y = 0,
      vx = 0,
      vy = 0,
      size = 16,
      health = 15,
      rooted = false,
      sprite = SPRITES.Enemy3,
      Shoot = function(self, dir)
        ESHOTS[#ESHOTS + 1] = {
          sprite = Copy(SPRITES.Enemy3Shot),
          alive = true,
          x = self.x,
          y = self.y,
          dir = dir,
          size = 8,
          vx = math.cos(dir + 0.1 * (math.random() - 0.5) * 2) * 200,
          vy = math.sin(dir + 0.1 * (math.random() - 0.5) * 2) * 200,
          shape = TRIANGLE,
          Update = function(self, dt)
            self.x = self.x + self.vx * dt
            self.y = self.y + self.vy * dt
            if not OnScreen(self.x, self.y) then self.alive = false end
          end,
          Draw = function(self)
            self.sprite:Draw(math.floor(self.x), math.floor(self.y), dir)
          end
        }
      end,
      Update = function(self, dt)
        self.shottimer = self.shottimer - 1

        self.vx = self.vx * 0.9
        self.vy = self.vy * 0.9

        self.x = self.x + self.vx * dt
        self.y = self.y + self.vy * dt

        if (lume.distance(self.x, self.y, player.x, player.y) > 150 or not OnScreen(self.x, self.y)) and not self.rooted then
          local dx = player.x - self.x
          local dy = player.y - self.y
          local dir = math.atan2(dy, dx)
          self.vx = self.vx + math.cos(dir) * 400 * dt
          self.vy = self.vy + math.sin(dir) * 400 * dt
        else
          self.rooted = true
          if self.shottimer <= 0 then
            local shot = CB.delay(function() self:Shoot(0) end, 0)

            for i = 0, 10 do
              shot = shot:after(function() self:Shoot((i * 31.4) / 180 * math.pi) end, 0.05)
            end

            self.shottimer = 240
          end
        end
      end,
      Draw = function(self)
        self.sprite:Draw(math.floor(self.x), math.floor(self.y))
      end
    }
  }

  FOES = {}
  PARTICLES = {}

  dfps = 0
  fps = 0
  secondtimer = 0
  love.graphics.setDefaultFilter("nearest")
  require("res.resources")

  ResetPlayer()

  gameCamera = Camera()
  World = HC(100)
  player:Register()
  debug = { colliders = false }

  SPAWN_TIMER = 2
  SPAWN_TIME = SPAWN_TIMER
end

function love.load()
  love.window.setMode(1024, 768, {msaa = 0})
  love.window.setTitle("LD35 - Shapeshift")

  CB = require "lib.tick"

  TRAIL = {}
  WALL = {}

  CIRCLE = 0
  SQUARE = 1
  TRIANGLE = 2

  BGCOLORS = {
    [CIRCLE] = {61, 64, 44, 255},
    [SQUARE] = {66, 46, 46, 255},
    [TRIANGLE] = {30, 37, 43, 255}
  }

  SPRITES = {
    [CIRCLE] = Sprite(love.graphics.newImage("res/playerCircle.png")),
    [SQUARE] = Sprite(love.graphics.newImage("res/player.png")),
    [TRIANGLE] = Sprite(love.graphics.newImage("res/playerTriangle.png")),
    CircleShot = Sprite(love.graphics.newImage("res/circleShot.png")),
    SquareShot = Sprite(love.graphics.newImage("res/squareShot.png")),
    TriangleShot = Sprite(love.graphics.newImage("res/triangleShot.png")),
    Enemy1 = Sprite(love.graphics.newImage("res/enemy1.png")),
    Enemy1Shot = Sprite(love.graphics.newImage("res/enemy1shot.png")),
    Enemy2 = Sprite(love.graphics.newImage("res/enemy2.png")),
    Enemy2Shot = Sprite(love.graphics.newImage("res/enemy2shot.png")),
    Enemy3 = Sprite(love.graphics.newImage("res/enemy3.png")),
    Enemy3Shot = Sprite(love.graphics.newImage("res/enemy3shot.png")),
    Particle = Sprite(love.graphics.newImage("res/particle.png")),
    Intro = Sprite(love.graphics.newImage("res/intro.png"))
  }

  SOUNDS = {
    [CIRCLE] = love.audio.newSource("res/triangleShot.wav"),
    [SQUARE] = love.audio.newSource("res/squareShot.wav"),
    [TRIANGLE] = love.audio.newSource("res/circleShot.wav"),
    Hit = love.audio.newSource("res/hit.wav"),
    Die = love.audio.newSource("res/die.wav")
  }

  STATE_START = 0
  STATE_GAME = 1
  STATE_END = 2

  STATE = STATE_START
end

function Contains(table, entry)
  for i, v in ipairs(table) do
    if v == entry then
      return true
    end
  end
end

function OnScreen(x, y)
  return (x < 1012 or x > 6) and
         (y < 754 or y > 6)
end

function love.update(dt)
  CB.update(dt)
  require("lib.lurker").update()

  if STATE == STATE_GAME then
    SPAWN_TIME = SPAWN_TIME - dt

    if SPAWN_TIME <= 0 then
      -- Spawn an enemy
      FOES[#FOES + 1] = Copy(ENEMIES[math.ceil(#ENEMIES * math.random() - 0.0001)])
      local vertical = math.random() > .5
      FOES[#FOES].x = vertical and (math.random() > .5 and -270 or 270) or (math.random() * 540 - 270)
      FOES[#FOES].y = (not vertical) and (math.random() > .5 and -200 or 200) or (math.random() * 400 - 200)
      SPAWN_TIME = 0.5 + SPAWN_TIMER - SPAWN_TIMER / 60
      SPAWN_TIMER = SPAWN_TIMER - SPAWN_TIMER / 60
    end

    for i, v in ipairs(FOES) do
      for j, k in ipairs(SHOTS) do
        if not k.time then
          if Hit(v.x, v.y, v.size, k.x, k.y, k.size) then
            Emitt(v.x, v.y, 1 + math.ceil(math.random() * 3))
            v.health = v.health - k.damage
            love.audio.play(SOUNDS.Hit)
            k.alive = false
          end
        end
      end

      v:Update(dt)

      if v.health <= 0 then
        Emitt(v.x, v.y, 5 + math.ceil(math.random() * 10))
        gameCamera:Shake(5)
        SCORE = SCORE + 1
        love.audio.play(SOUNDS.Die)
        table.remove(FOES, i)
      end
    end

    for i, v in ipairs(PARTICLES) do
      v:Update(dt)

      if math.max(math.abs(v.vx), math.abs(v.vy)) < 10 then
        table.remove(PARTICLES, i)
      end
    end

    for i, v in ipairs(ESHOTS) do
      v:Update(dt)

      for j, k in ipairs(WALL) do
        if Hit(v.x, v.y, v.size, k[1], k[2], 12) then
          v.alive = false
        end
      end

      if v.alive == true and player.form ~= v.shape and Hit(v.x, v.y, v.size, player.x, player.y, player.size) and player.candie == true and player.dead == false then
        player.dead = true
        player.form = SQUARE
        player.formFrames = 0
        Emitt(player.x, player.y, 30)
        love.audio.play(SOUNDS.Die)
        CB.delay(function() STATE = STATE_END end, 3)
      end

      if v.alive == false then
        table.remove(ESHOTS, i)
      end
    end

    if player.dead == false then
      local lr = false
      if love.keyboard.isDown('d', "right") then
        player.vx = player.vx + 1700 * dt
        lr = true
      elseif love.keyboard.isDown('a', "left") then
        player.vx = player.vx - 1700 * dt
        lr = true
      end

      local speed = 1700
      if lr == true then speed = speed / math.sqrt(2) end
      if love.keyboard.isDown('w', "up") then
        player.vy = player.vy - speed * dt
      elseif love.keyboard.isDown('s', "down") then
        player.vy = player.vy + speed * dt
      end

      if love.keyboard.isDown('q') and player.form ~= CIRCLE then
        player.lastForm = player.form
        player.formFrames = 0
        player.form = CIRCLE
      end

      if love.keyboard.isDown('e') and player.form ~= SQUARE then
        player.lastForm = player.form
        player.formFrames = 0
        player.form = SQUARE
      end

      if love.keyboard.isDown('r') and player.form ~= TRIANGLE then
        player.lastForm = player.form
        player.formFrames = 0
        player.form = TRIANGLE
      end

      ABILITIES[player.form](player)
      SHOOT[player.form](player)

      player.formFrames = player.formFrames + 1
      player:Update(dt)
    end

    gameCamera:Update(player.x, player.y)

    for i, v in ipairs(SHOTS) do
      v:Update(dt)
      if v.alive == false then
        table.remove(SHOTS, i)
      end
    end

    secondtimer = secondtimer + dt

    if secondtimer >= 1 then
      dfps = fps
      fps = 0
      secondtimer = 0
    end

    -- debug
  elseif STATE == STATE_START then
    if love.keyboard.isDown(' ') then
      Setup()
      STATE = STATE_GAME
    end
  else
    if love.keyboard.isDown('return') then
      STATE = STATE_START
    end
  end
end

function love.draw()
  if STATE == STATE_GAME then
    if player.formFrames < 8 then
      local factor = player.formFrames / 8
      local r = lume.smooth(BGCOLORS[player.lastForm][1], BGCOLORS[player.form][1], factor)
      local g = lume.smooth(BGCOLORS[player.lastForm][2], BGCOLORS[player.form][2], factor)
      local b = lume.smooth(BGCOLORS[player.lastForm][3], BGCOLORS[player.form][3], factor)
      love.graphics.setColor(r, g, b)
    else
      love.graphics.setColor(BGCOLORS[player.form])
    end

    love.graphics.rectangle('fill', 0, 0, 1024, 768)
    love.graphics.setColor(255, 255, 255)

    gameCamera:Use()

    fps = fps + 1
    if player.dead == false then
      player:Draw()
    end

    for i, v in ipairs(TRAIL) do
      v[3] = v[3] - 1

      love.graphics.setColor(255, 255, 255, 80)
      SPRITES[player.form]:Draw(v[1], v[2])
      love.graphics.setColor(255, 255, 255, 255)

      if v[3] <= 0 then
        table.remove(TRAIL, i)
      end
    end

    for i, v in ipairs(WALL) do
      v[3] = v[3] - 1

      if v[3] <= 10 then
        love.graphics.setColor(255, 255, 255, 255 * v[3] / 10)
      end
      SPRITES.CircleShot:Draw(v[1], v[2])
      love.graphics.setColor(255, 255, 255, 255)

      if v[3] <= 0 then
        table.remove(WALL, i)
      end
    end

    for i, v in ipairs(PARTICLES) do
      v:Draw()
    end

    for i, v in ipairs(SHOTS) do
      v:Draw()
    end


    for i, v in ipairs(ESHOTS) do
      v:Draw()
    end

    for i, v in ipairs(FOES) do
      v:Draw()
    end

    if player.dead == false then
      if player.formFrames < 8 then
        local factor = player.formFrames / 8
        local lastA = lume.lerp(255, 0, factor)
        local newA = lume.smooth(0, 255, factor)
        love.graphics.setColor(255, 255, 255, lastA)
        SPRITES[player.lastForm]:Draw(math.floor(player.x), math.floor(player.y))
        love.graphics.setColor(255, 255, 255, newA)
        SPRITES[player.form]:Draw(math.floor(player.x), math.floor(player.y))
        love.graphics.setColor(255, 255, 255)
      else
        SPRITES[player.form]:Draw(math.floor(player.x), math.floor(player.y))
      end
    end

    gameCamera:Unuse()
    --love.graphics.setNewFont(12)
    --love.graphics.print("Fps: " .. dfps, 43, 10)
    love.graphics.setNewFont(48)
    love.graphics.print("" .. SCORE, 10, 10)
  elseif STATE == STATE_START then
    love.graphics.setColor(BGCOLORS[SQUARE])
    love.graphics.rectangle('fill', 0, 0, 1024, 768)
    love.graphics.setColor(255, 255, 255)
    SPRITES.Intro:Draw(1024 / 2, 768 / 2)
  else
    love.graphics.setColor(BGCOLORS[SQUARE])
    love.graphics.rectangle('fill', 0, 0, 1024, 768)
    love.graphics.setColor(255, 255, 255)
    love.graphics.setNewFont(48)
    love.graphics.print("You died.", 240, 50)
    love.graphics.print("Score: " .. SCORE, 240, 98)

    love.graphics.setNewFont(18)
    love.graphics.print("Press enter to continue...", 240, 500)
  end
end

function Emitt(x, y, num)
  for i = 0, num do
    PARTICLES[#PARTICLES + 1] = {
      sprite = Copy(SPRITES.Particle),
      x = x,
      y = y,
      vx = (math.random() - 0.5) * 2 * 600,
      vy = (math.random() - 0.5) * 2 * 600,
      scale = 1 + math.random(),
      Update = function(self, dt)
        self.vx = self.vx * 0.9
        self.vy = self.vy * 0.9
        self.x = self.x + self.vx * dt
        self.y = self.y + self.vy * dt
      end,
      Draw = function(self)
        love.graphics.setColor(255, 255, 255, 255 * lume.clamp(math.max(math.abs(self.vx), math.abs(self.vy)) / 100, 0, 1))
        self.sprite:Draw(self.x, self.y, 0, self.scale, self.scale)
        love.graphics.setColor(255, 255, 255, 255)
      end
    }
  end
end

function ResetPlayer()
  player = Entity(0, 0, 12, 12, 0, 0)
  player.sprite = Sprite(love.graphics.newImage("res/player.png")) -- Image, frameSize(x, y), rows, cols, frameTimes
  player.id = "Player"
  player.size = 12

  player.form = SQUARE
  player.lastForm = CIRCLE
  player.formFrames = 0
  player.atimer = 0
  player.shottimer = 0
  player.dead = false

  SCORE = 0
end
