require("lib.class")
require("tilemap")
require("lib.file")
local lume = require("lib.lume")

local HC = require("lib.hardoncollider")

Map = class()

function Map:_init()
  self.x = 1
  self.y = 1
  self:Reload()

  -- Lerp stuff
  self.transSpeed = 1
  self.transitionFactor = 0
  self.transitioning = false
  self.toX = 0
  self.toY = 0
  self.toP = 0
  self.pStartX = 0
  self.pStartY = 0
  self.transX = 0
  self.transY = 0
  self.destX = 0
  self.destY = 0
  self.transOX = 0
  self.transOY = 0
  self.transWS = 1
  self.transHS = 1
  self.colliders = {}
  self:Move(1, 1, "spawn")
  self.entities = {}
  self.nextEntities = {}
end

function Map:Move(x, y, portal)
  self.toX = x
  self.toY = y
  self.toP = portal
  self.transitioning = true
  self.transitionFactor = 0
  self.pStartX = player.x
  self.pStartY = player.y

  local dest = self.map[self.toX][self.toY]
  local destP = dest.arrives[self.toP]

  self.transOX = 0
  self.transOY = 0
  self.destX, self.destY = 0, 0

  self.transOY = player.y - destP.y

  local hs = 1
  if currentMap.heightPixels > dest.heightPixels then
    hs = currentMap.heightPixels / dest.heightPixels
  end

  if destP.y >= dest.heightPixels - 120 then
    self.transOY = -dest.heightPixels + 240 * hs
  end

  if destP.y <= 120 then
    --self.transOY = 240 * hs
  end

  if x > self.x then
    self.destX = -currentMap.widthPixels
  elseif x < self.x then
    self.destX = currentMap.widthPixels
  elseif y > self.y then
    self.destY = -currentMap.heightPixels
    self.transOY = 0
  elseif y < self.y then
    self.destY = currentMap.heightPixels
    self.transOY = 0
  end

  if currentMap.widthPixels > dest.widthPixels and x ~= 0 then
    self.destX = self.destX / (currentMap.widthPixels / dest.widthPixels)
  end

  self.nextEntities = Copy(dest.entities)
end

function Map:Update(dt)
  if self.transitioning then
    local dest = self.map[self.toX][self.toY]
    self.transitionFactor = self.transitionFactor + self.transSpeed * dt

    if self.transOX == 0 then
      player.x = lume.smooth(self.pStartX, dest.arrives[self.toP].x, self.transitionFactor)
    end

    if self.transOY == 0 then
      player.y = lume.smooth(self.pStartY, dest.arrives[self.toP].y, self.transitionFactor)
    end

    self.transX = lume.smooth(0, self.destX, self.transitionFactor)
    self.transY = lume.smooth(0, self.destY, self.transitionFactor)

    if self.transitionFactor >= 1 then
      self.transitioning = false
      self.transX = 0
      self.transY = 0
      currentMap = self.map[self.toX][self.toY]
      self.x = self.toX
      self.y = self.toY

      if self.transOY ~= 0 then
        player.y = currentMap.arrives[self.toP].y
      end

      if self.transOX ~= 0 then
        player.x = currentMap.arrives[self.toP].x
      end

      self.colliders = {}
      blood = {}

      World:resetHash()
      World = HC()
      self.entities = self.nextEntities
      for i, v in ipairs(self.entities) do
        v:Register()
      end

      for index, box in ipairs(currentMap:GetObjectGroup("collision").objects) do
        self.colliders[#self.colliders + 1] = World:rectangle(box.x, box.y, box.width, box.height)
      end
      player:Register()
    end
  else
    for i, v in ipairs(self.entities) do
      v:Update(dt)
      if v.dead == true then
        v:Remove()
        table.remove(self.entities, i)
      end
    end
  end
end

function Map:Draw(tileset)
  currentMap:Draw(tileset, lume.round(self.transX), lume.round(self.transY))
  for i, v in ipairs(self.entities) do
    v:Draw(lume.round(self.transX), lume.round(self.transY))
  end

  if debug.colliders == true then
    for i, v in ipairs(self.colliders) do
      v:draw('line')
    end
  end

  if self.transitioning then
    self.map[self.toX][self.toY]:Draw(tileset, lume.round(self.transX - self.destX + self.transOX), lume.round(self.transY - self.destY + self.transOY))
  end
end

function Map:DrawGlow(tileset)
  currentMap:DrawGlow(tileset, lume.round(self.transX), lume.round(self.transY))

  if self.transitioning then
    self.map[self.toX][self.toY]:DrawGlow(tileset, lume.round(self.transX - self.destX + self.transOX), lume.round(self.transY - self.destY + self.transOY))
  end
end

function Map:Reload()
  local levelFiles = ScanDir([[.\src\res\map\*.lua]])
  self.map = {}
  for i, v in ipairs(levelFiles) do
    local posDelim = string.find(v, "-")-1
    local mapX = tonumber(string.sub(v, 0, posDelim))
    local mapY = tonumber(string.sub(v, posDelim+2, string.len(v) - posDelim-3))
    local reqName = v--string.sub(v, 0, string.find(v, ".lua")-1)

    if not self.map[mapX] then
      self.map[mapX] = {}
    end

    print("Adding map: " .. mapX .. " " .. mapY)
    self.map[mapX][mapY] = Tilemap("src/res/map/" .. reqName)
  end
  currentMap = self.map[self.x][self.y]
end
