require("lib.class")

Sprite = class()

function Sprite:_init(image, width, height, rows, cols, animationtime)
  self.image = image
  image:setFilter("nearest", "nearest")
  self.frameWidth = width or image:getWidth()
  self.frameHeight = height or image:getHeight()
  self.width = image:getWidth()
  self.height = image:getHeight()
  self.rows = rows or self.height / self.frameHeight
  self.cols = cols or self.width / self.frameWidth
  self.frames = self.rows * self.cols

  if type(animationtime) == 'number' then
    self.animated = true
    self.variedTime = false
    self.frameTime = animationtime
  elseif type(animationtime) == 'table' then
    self.animated = true
    self.variedTime = true
    self.frameTime = animationtime
    self.frames = #animationtime
  else
    self.animated = false
  end

  self.currentFrame = 1
  self.currentTime = 0
  self.loops = 0

  local currentFrameX = 0
  local currentFrameY = 0
  self.quads = {}
  for i = 1, self.frames do
    self.quads[#self.quads + 1] = love.graphics.newQuad(currentFrameX * self.frameWidth, currentFrameY * self.frameHeight, self.frameWidth, self.frameHeight, self.width, self.height)
    currentFrameX = currentFrameX + 1
    if currentFrameX > self.cols - 1 then
      currentFrameX = 0
      currentFrameY = currentFrameY + 1
    end
  end
end

function Sprite:Update(dt)
  self.currentTime = self.currentTime + dt

  if self.animated and self.variedTime then
    if self.currentTime >= self.frameTime[self.currentFrame] then
      self.currentFrame = self.currentFrame + 1
      self.currentTime = 0
    end
  elseif self.animated then
    if self.currentTime >= self.frameTime then
      self.currentFrame = self.currentFrame + 1
      self.currentTime = 0
    end
  end

  if self.currentFrame > self.frames then
    self.currentFrame = 1
    self.loops = self.loops + 1
  end
end

function Sprite:Draw(x, y, rot, sx, sy, frame)
  local scx = sx or 1
  local scy = sy or scx
  local r = rot or 0
  local f = frame or self.currentFrame
  if self.animated or frame then
    love.graphics.draw(self.image, self.quads[f], x, y, r, scx, scy, self.frameWidth / 2, self.frameHeight / 2)
  else
    love.graphics.draw(self.image, x, y, r, scx, scy, self.frameWidth / 2, self.frameHeight / 2)
  end
end
