require("lib.class")
require("collision")
require("util")

Tilemap = class()

function Tilemap:_init(file)
  local map = dofile(file)
  self.width = map.width
  self.height = map.height
  self.tileWidth = map.tilewidth
  self.tileHeight = map.tileheight
  self.widthPixels = self.width * self.tileWidth
  self.heightPixels = self.height * self.tileHeight
  self.arrives = {}
  self.portals = {}
  self.interacts = {}
  self.entities = {}

  self.layers = {}
  for i, v in pairs(map.layers) do
    self.layers[i] = {}
    self.layers[i].name = v.name

    if v.type == "tilelayer" then
      self.layers[i].type = "tile"
      self.layers[i].tiles = {}
      self.layers[i].name = v.name
      for j, k in ipairs(v.data) do
        if k ~= 0 then
          local ins = {}
          ins.x = 0.5 + (j-1) % self.width
          ins.y = 0.5 + math.floor((j-1) / self.width)
          --if ins.x == 0.5 then ins.y = ins.y - 1 end
          ins.tileID = k
          self.layers[i].tiles[#self.layers[i].tiles + 1] = ins
        end
      end
    elseif v.type == "objectgroup" then
      self.layers[i].type = "object"
      self.layers[i].objects = {}
      self.layers[i].name = v.name
      for j, k in ipairs(v.objects) do
        local ins = {}
        ins.x = math.floor(k.x)
        ins.y = math.floor(k.y)
        ins.width = math.floor(k.width)
        ins.height = math.floor(k.height)
        ins.name = k.name
        self.layers[i].objects[#self.layers[i].objects + 1] = ins
      end
    end
  end

  for i, v in ipairs(self.layers) do
    if v.type == "object" then
      if v.name == "portal" then
        for j, k in ipairs(v.objects) do
          if string.sub(k.name, 1, 2) == "to" then
            local transportCode = string.sub(k.name, string.find(k.name, "o")+1, 1000)
            local x = string.sub(transportCode, 0, string.find(transportCode, ":")-1)
            transportCode = string.sub(transportCode, string.find(transportCode, ":")+1, 1000)
            local y = string.sub(transportCode, 0, string.find(transportCode, ":")-1)
            transportCode = string.sub(transportCode, string.find(transportCode, ":")+1, 1000)
            local portal = transportCode

            self.portals[#self.portals + 1] = {
              x = k.x,
              y = k.y,
              width = k.width,
              height = k.height,
              toX = tonumber(x),
              toY = tonumber(y),
              toPortal = portal
            }
          else
            self.arrives[k.name] = {
              x = k.x,
              y = k.y
            }
          end
        end
      elseif v.name == "interactions" then
        for j, k in ipairs(v.objects) do
          self.interacts[#self.interacts + 1] = {
            x = k.x,
            y = k.y,
            width = k.width,
            height = k.height,
            code = k.name,
          }
        end
      elseif v.name == "entities" then
        for j, k in ipairs(v.objects) do
          if string.match(k.name, "%(") then
            -- If the string contains a (, run the name as code.
            local f = loadstring("return " .. k.name)
            self.entities[#self.entities + 1] = f()
          else
            -- Else, spawn an entity of v.name at v's position.
            local str = "return " .. k.name .. "(" .. k.x .. ", " .. k.y .. ")"
            local f = loadstring(str)
            --self.entities[#self.entities + 1] = f()
          end
        end
      end
    end
  end
end

function Tilemap:Draw(sprite, offX, offY)
  local x = offX or 0
  local y = offY or 0
  for i, v in ipairs(self.layers) do
    if v.type == "tile" and v.name ~= "glow" then
      for j, k in ipairs(v.tiles) do
        sprite:Draw(x + k.x * self.tileWidth, y + k.y * self.tileHeight, 0, 1, 1, k.tileID)
      end
    end
  end
end

function Tilemap:DrawGlow(sprite, offX, offY)
  local x = offX or 0
  local y = offY or 0
  for i, v in ipairs(self.layers) do
    if v.type == "tile" and v.name == "glow" then
      for j, k in ipairs(v.tiles) do
        sprite:Draw(x + k.x * self.tileWidth, y + k.y * self.tileHeight, 0, 1, 1, k.tileID)
      end
    end
  end
end

function Tilemap:GetObjectGroup(name)
  for i, v in ipairs(self.layers) do
    if v.type == "object" and v.name == name then
      return v
    end
  end
end

function Tilemap:GetObjectFromGroup(group, name)
  local g = self:GetObjectGroup(group)
  for i, v in ipairs(g.objects) do
    if v.name == name then
      return v
    end
  end
end

function Tilemap:PointCollides(x, y, group)
  local g = self:GetObjectGroup(group)
  for i, v in ipairs(g.objects) do
    if RectangleIntersect(x, y, 1, 1, v.x, v.y, v.width, v.height) then
      return true
    end
  end

  return false
end

function Tilemap:RectCollides(x, y, w, h, group)
  local g = self:GetObjectGroup(group)
  for i, v in ipairs(g.objects) do
    if RectangleIntersect(x, y, w, h, v.x, v.y, v.width, v.height) then
      return true
    end
  end

  return false
end

function Tilemap:HandleMove(x, y)
  for i, v in ipairs(self.portals) do
    if RectangleIntersect(x, y, 1, 1, v.x, v.y, v.width, v.height) then
      map:Move(v.toX, v.toY, v.toPortal)
      return true
    end
  end

  return false
end

function Tilemap:Interact(x, y)
  for i, v in ipairs(self.interacts) do
    if RectangleIntersect(x, y, 1, 1, v.x, v.y, v.width, v.height) then
      local f = loadstring(v.code)
      f()
    end
  end
end
